unit Screen;
{
  SCREEN.TPU
  Programmed by Boo Khan Ming

  Provides necessary screen elements to simplify the design of user interface
  and data entry.


  DEFINITION OF PROCEDURES AND FUNCTIONS
  ======================================

  procedure SaveScreen(FramePtr:byte);

  Purpose: Save current text screen to stack with all characters and colors
           retained.
  Arguments:FramePtr=Index number to store the screen image
                     (1 to 2 only, as limited by MaxScreenFrame)

  -------------------------------------------------------------------------

  procedure RestoreScreen(FramePtr:byte);

  Purpose:  Restore previously saved screen back to video buffer.
            All information on screen will be overwritten immediately.
  Arguments:FramePtr=Index number to retrieve the screen image
                     (1 to 2 only, as limited by MaxScreenFrame)
  Notes:    Both SaveScreen and RestoreScreen must point to the same
            index number!
  Pros:     Fast screen saving and loading without the needs to redraw screen
            and clear screen manually.
            Allows storage and retrieval of multiple screen images.

  -------------------------------------------------------------------------

  procedure PrepareTextScreen;

  Purpose:  Must be called at MAIN program startup ONCE ONLY!!
            Performs all necessary things like clear screen and enable
            background high intensity.
            NO clear screen command is needed.

  -------------------------------------------------------------------------

  procedure ShutTextScreen;

  Purpose:  Corressponds to PrepareTextScreen command, called at the end
            of main program ONCE ONLY.
            Performs all necessary disposal.
            NO clear screen command is needed.

  -------------------------------------------------------------------------

  procedure MakeBox(x1,y1,x2,y2,ForeColor,BackColor:byte);

  Purpose:  Make a singler border rectangular box with specified size and
            position.
  Arguments:x1=Left
            y1=Top
            x2=Right
            y2=Bottom
            ForeColor=Foreground color
            BackColor=Background color

  -------------------------------------------------------------------------

  procedure MakeDialog(x1,y1,x2,y2,TitleForeColor,TitleBackColor,
                       ForeColor,BackColor:byte;Title:string);

  Purpose:  Used internally (but may be called for customization)

  -------------------------------------------------------------------------

  procedure MakeField(x1,y1,Len:byte;FieldName:string);

  Purpose:  Used internally (but may be called for customization)

  -------------------------------------------------------------------------

  procedure OpenScreen(Title:string);

  Purpose:  Should be called whenever a new main subject/topic entered.
            OpenDialog should be called for subtopic instead of this.
            Print the title on the top and draw border around screen.
            Color predefined (can be altered at CONST declaration area)

  -------------------------------------------------------------------------

  procedure OpenDialog(x1,y1,x2,y2:byte;Title:string);

  Purpose:  For displaying a smaller area window for information and menu
            purposes.

  -------------------------------------------------------------------------

  procedure OpenErrorDialog(x1,y1,x2,y2:byte;Title:string);

  Purpose:  For displaying a smaller area window for reporting error.

  -------------------------------------------------------------------------

  procedure OpenWarningDialog(x1,y1,x2,y2:byte;Title:string);

  Purpose:  For displaying a smaller area window for prompting user for
            confirmation.

  -------------------------------------------------------------------------

  function PopUpMenu(PopUpMenuItemArray:PopUpMenuArrayType;
                     PopUpMenuItemCount:byte):byte;

  Purpose:  Auto-building a pop-up style menu.
            Suitable dialog in size must be opened first in order to
            arrange and display the menu options for selection
  Arguments:PopUpMenuItemArray=Array of string with each consist of the
                               name of menu items
                               Assigned at VAR declaration area to
                               PopUpMenuArrayType
                               (eg. var MyAppMenu:PopUpMenuArrayType)
            PopUpMenuItemCount=Total number of menu items
  Specials: Use empty line as menu separator
            Use symbol (~) to precede menu item string that you want to
            disable (eg. "Exit to DOS" to "~Exit to DOS")
  Return:   255=Cancel
            Others=Selected choice number (1 for array item 1, and so forth)

  -------------------------------------------------------------------------

  function AcquireString(x1,y1,Len:byte;FieldName:string;
                         var FieldValue:string):boolean;
  function AcquireWord(x1,y1,Len:byte;FieldName:string;
                       var FieldValue:word):boolean;
  function AcquireReal(x1,y1,Len:byte;FieldName:string;
                       var FieldValue:real):boolean;

  Purpose:  Simplify the process of input and validation as well.
            Just select the appropriate data type for input, for instance,

              To get a number(word) from user, use AcquireWord
              To get a name(string) from user, use AcquireString
              To get a balance(real) from user, use AcquireReal

            Type of Char or Integer are disallowed as they are not much
            needed in the program. Word is preferred than Integer because
            there will no number of CD or people in negative value.

  Arguments:x1=Location of field from left
            y1=Location of field from top
            Len=Limit the size of input (eg. Len=3 if only 3 digits required)
            FieldName=Caption/Label for the subject of the field (eg. Cost)
            FieldValue=Contains the predefined value or empty value

  Return:   True=OK
            False=Cancelled
            FieldValue=Contains the latest value entered by user
}

interface

const
  MaxMenuItem=20;
  MaxScreenFrame=2;

  TitleBarFrontColor=15;
  TitleBarBackColor=1;
  SubTitleBarFrontColor=15;
  SubTitleBarBackColor=3;
  DesktopFrontColor=7+16;
  DesktopBackColor=0;
  DialogBarFrontColor=15+16;
  DialogBarBackColor=1;
  DialogTextFrontColor=0;
  DialogTextBackColor=7;
  DialogHelpFrontColor=8;
  DialogHelpBackColor=7;
  MenuFrontColor=DialogTextFrontColor;
  MenuBackColor=DialogTextBackColor;
  SelectionBarFrontColor=7;
  SelectionBarBackColor=1;
  InputFrontColor=7;
  InputBackColor=0;
  ErrorFrontColor=15+16;
  ErrorBackColor=4;
  AlertFrontColor=0+16;
  AlertBackColor=6;
  FieldFrontColor=7;
  FieldBackColor=0;

type
  PopUpMenuArrayType=array [1..MaxMenuItem] of string[80];

procedure SaveScreen(FramePtr:byte);
procedure RestoreScreen(FramePtr:byte);

procedure PrepareTextScreen;
procedure ShutTextScreen;

procedure MakeBox(x1,y1,x2,y2,FrontColor,BackColor:byte);
procedure MakeDialog(x1,y1,x2,y2,TitleFrontColor,TitleBackColor,
                     FrontColor,BackColor:byte;Title:string);
procedure MakeField(x1,y1,Len:byte;FieldName:string);

procedure OpenScreen(Title:string);
procedure OpenDialog(x1,y1,x2,y2:byte;Title:string);
procedure OpenErrorDialog(x1,y1,x2,y2:byte;Title:string);
procedure OpenWarningDialog(x1,y1,x2,y2:byte;Title:string);

function PopUpMenu(PopUpMenuItemArray:PopUpMenuArrayType;
                   PopUpMenuItemCount:byte):byte;

function AcquireString(x1,y1,Len:byte;FieldName:string;
                       var FieldValue:string):boolean;
function AcquireWord(x1,y1,Len:byte;FieldName:string;
                     var FieldValue:word):boolean;
function AcquireReal(x1,y1,Len:byte;FieldName:string;
                     var FieldValue:real):boolean;

implementation

uses Crt;

type
  ScreenType=array [1..2000] of word;

var
  ScreenPtr:^ScreenType;
  ScreenBuffer:array [1..MaxScreenFrame] of ScreenType;
  FramePtr:byte;

procedure SaveScreen(FramePtr:byte);
begin
  ScreenPtr:=Ptr($b800,0);
  ScreenBuffer[FramePtr]:=ScreenPtr^;
end;

procedure RestoreScreen(FramePtr:byte);
begin
  ScreenPtr:=Ptr($b800,0);
  ScreenPtr^:=ScreenBuffer[FramePtr];
end;

procedure PrepareTextScreen;
begin
  CheckBreak:=False;
  DirectVideo:=True;

  TextMode(co80);

  asm
    mov ax,1003h
    xor bx,bx
    int 10h
  end;
end;

procedure ShutTextScreen;
begin
  TextMode(LastMode);
end;

procedure MakeBox(x1,y1,x2,y2,FrontColor,BackColor:byte);
var
  b:byte;

begin
  Window(1,1,80,25);
  TextColor(FrontColor);
  TextBackground(BackColor);

  for b:=x1 to x2 do
  begin
    GotoXY(b,y1);
    Write(#219);
    GotoXY(b,y2);
    Write(#219);
  end;

  for b:=y1 to y2 do
  begin
    GotoXY(x1,b);
    Write(#219);
    GotoXY(x2,b);
    Write(#219);
  end;

  GotoXY(x1,y1);
  Write(#219);
  GotoXY(x1,y2);
  Write(#219);
  GotoXY(x2,y1);
  Write(#219);
  GotoXY(x2,y2);
  Write(#219);

  Window(x1+1,y1+1,x2-1,y2-1);
end;

procedure MakeDialog(x1,y1,x2,y2,TitleFrontColor,TitleBackColor,
                     FrontColor,BackColor:byte;Title:string);
begin
  Window(x1,y1,x2,y2);
  TextColor(TitleFrontColor);
  TextBackground(TitleBackColor);
  ClrScr;

  Window(x1+1,y1+1,x2-1,y1+1);
  Write(Title);

  Window(x1+1,y1+2,x2-1,y2-1);
  TextColor(FrontColor);
  TextBackground(BackColor);
  ClrScr;

  Window(x1+2,y1+3,x2-2,y2-2);
end;

procedure OpenScreen(Title:string);
begin
  MakeDialog(1,1,80,25,TitleBarFrontColor,TitleBarBackColor,
             DesktopFrontColor,DesktopBackColor,Title);
end;

procedure OpenDialog(x1,y1,x2,y2:byte;Title:string);
begin
  MakeDialog(x1,y1,x2,y2,DialogBarFrontColor,DialogBarBackColor,
             DialogTextFrontColor,DialogTextBackColor,Title);
end;

procedure OpenErrorDialog(x1,y1,x2,y2:byte;Title:string);
begin
  MakeDialog(x1,y1,x2,y2,ErrorFrontColor,ErrorBackColor,
             DialogTextFrontColor,DialogTextBackColor,Title);
end;

procedure OpenWarningDialog(x1,y1,x2,y2:byte;Title:string);
begin
  MakeDialog(x1,y1,x2,y2,AlertFrontColor,AlertBackColor,
             DialogTextFrontColor,DialogTextBackColor,Title);
end;

function PopUpMenu(PopUpMenuItemArray:PopUpMenuArrayType;
                   PopUpMenuItemCount:byte):byte;
var
  b:byte;
  c:char;
  NewChoice,OldChoice:byte;

  procedure UpdatePopUpMenu;
  begin
    if NewChoice<>OldChoice then
    begin
      TextColor(MenuFrontColor);
      TextBackground(MenuBackColor);

      GotoXY(1,OldChoice);
      Write(PopUpMenuItemArray[OldChoice]);

      TextColor(SelectionBarFrontColor);
      TextBackground(SelectionBarBackColor);

      GotoXY(1,NewChoice);
      Write(PopUpMenuItemArray[NewChoice]);
    end;
  end;

begin
  TextColor(MenuFrontColor);
  TextBackground(MenuBackColor);
  ClrScr;

  for b:=1 to PopUpMenuItemCount do
  begin
    GotoXY(1,b);
    if PopUpMenuItemArray[b][1]='~' then
      Write(Copy(PopUpMenuItemArray[b],2,Length(PopUpMenuItemArray[b])))
    else
      Write(PopUpMenuItemArray[b]);
  end;

  PopUpMenu:=255;
  c:=#0;
  NewChoice:=1;
  OldChoice:=2;

  UpdatePopUpMenu;

  repeat
    if Keypressed then
    begin
      OldChoice:=NewChoice;

      c:=ReadKey;
      case c of
        #72:{Dec(NewChoice);}
        { Optional code for adaption of menu separator and item disabling}
        begin
          Dec(NewChoice);

          while ((PopUpMenuItemArray[NewChoice]='') or (PopUpMenuItemArray[NewChoice][1]='~'))
          and (NewChoice>1) do
            Dec(NewChoice);
        end;
        #80:{Inc(NewChoice);}
        { Optional code for adaption of menu separator and item disabling }
        begin
          Inc(NewChoice);

          while ((PopUpMenuItemArray[NewChoice]='') or (PopUpMenuItemArray[NewChoice][1]='~')) do
            Inc(NewChoice);
        end;
        #71:NewChoice:=1;
        #79:NewChoice:=PopUpMenuItemCount;
        #13:
        begin
          PopUpMenu:=NewChoice;
          Exit;
        end;
      end;

      if NewChoice<1 then
        NewChoice:=PopUpMenuItemCount;
      if NewChoice>PopUpMenuItemCount then
        NewChoice:=1;

      UpdatePopUpMenu;
    end;
  until c=#27;
end;

procedure MakeField(x1,y1,Len:byte;FieldName:string);
begin
  Window(1,1,80,25);

  TextColor(DialogTextFrontColor);
  TextBackground(DialogTextBackColor);

  GotoXY(x1,y1);
  Write(FieldName);

  TextColor(FieldFrontColor);
  TextBackground(FieldBackColor);

  Write(#32:Len);
end;

function AcquireString(x1,y1,Len:byte;FieldName:string;
                       var FieldValue:string):boolean;
var
  b:byte;
  c:char;
  s:string;

  procedure UpdateField;
  begin
    Window(x1+Length(FieldName),y1,x1+Length(FieldName)+Len-1,y1);
    TextColor(FieldFrontColor);
    TextBackground(FieldBackColor);
    ClrScr;

    Window(x1+Length(FieldName),y1,x1+Length(FieldName)+Len,y1);
    Write(s);
  end;

begin
  MakeField(x1,y1,Len,FieldName);
  Window(x1+Length(FieldName),y1,x1+Length(FieldName)+Len-1,y1);

  AcquireString:=False;
  s:=FieldValue;
  c:=#0;

  UpdateField;

  repeat
    if Keypressed then
    begin
      c:=ReadKey;
      case c of
        #32..#127:
        if WhereX<=Len then
        begin
          s:=s+c;
          UpdateField;
        end;
        #8:
        if WhereX>1 then
        begin
          s:=Copy(s,1,Length(s)-1);
          UpdateField;
        end;
        #13:
        if s<>'' then
        begin
          AcquireString:=True;
          FieldValue:=s;
          Exit;
        end;
      end;
    end;
  until c=#27;
end;

function AcquireWord(x1,y1,Len:byte;FieldName:string;
                     var FieldValue:word):boolean;
var
  b:byte;
  c:char;
  s:string;
  w:word;
  i:integer;

  procedure UpdateField;
  begin
    Window(x1+Length(FieldName),y1,x1+Length(FieldName)+Len-1,y1);
    TextColor(FieldFrontColor);
    TextBackground(FieldBackColor);
    ClrScr;

    Window(x1+Length(FieldName),y1,x1+Length(FieldName)+Len,y1);
    Write(s);
  end;

begin
  MakeField(x1,y1,Len,FieldName);
  Window(x1+Length(FieldName),y1,x1+Length(FieldName)+Len-1,y1);

  AcquireWord:=False;
  if FieldValue<>0 then
    Str(FieldValue,s)
  else
    s:='';
  c:=#0;

  UpdateField;

  repeat
    if Keypressed then
    begin
      c:=ReadKey;
      case c of
        '0'..'9':
        if WhereX<=Len then
        begin
          s:=s+c;
          UpdateField;
        end;
        #8:
        if WhereX>1 then
        begin
          s:=Copy(s,1,Length(s)-1);
          UpdateField;
        end;
        #13:
        if s<>'' then
        begin
          AcquireWord:=True;
          Val(s,w,i);
          FieldValue:=w;
          Exit;
        end;
      end;
    end;
  until c=#27;
end;

function AcquireReal(x1,y1,Len:byte;FieldName:string;
                     var FieldValue:real):boolean;
var
  b:byte;
  c:char;
  s:string;
  r:real;
  i:integer;

  procedure UpdateField;
  begin
    Window(x1+Length(FieldName),y1,x1+Length(FieldName)+Len-1,y1);
    TextColor(FieldFrontColor);
    TextBackground(FieldBackColor);
    ClrScr;

    Window(x1+Length(FieldName),y1,x1+Length(FieldName)+Len,y1);
    Write(s);
  end;

begin
  MakeField(x1,y1,Len,FieldName);
  Window(x1+Length(FieldName),y1,x1+Length(FieldName)+Len-1,y1);

  AcquireReal:=False;
  if FieldValue<>0 then
    Str(FieldValue:0:2,s)
  else
    s:='';
  c:=#0;

  UpdateField;

  repeat
    if Keypressed then
    begin
      c:=ReadKey;
      case c of
        '0'..'9','.':
        if WhereX<=Len then
        begin
          s:=s+c;
          UpdateField;
        end;
        #8:
        if WhereX>1 then
        begin
          s:=Copy(s,1,Length(s)-1);
          UpdateField;
        end;
        #13:
        if s<>'' then
        begin
          AcquireReal:=True;
          Val(s,r,i);
          FieldValue:=r;
          Exit;
        end;
      end;
    end;
  until c=#27;
end;

end.